@@ -1073,9 +1073,15 @@ gdm_test_opt (const char *cmd, const char *help, const char *option)
 int
 gdm_fdgetc (int fd)
 {
-	char buf[1];
+	unsigned char buf[1];
 	int bytes;
 
+	/*
+	 * Must used an unsigned char buffer here because the GUI sends
+	 * username/password data as utf8 and the daemon will interpret
+	 * any character sent with its high bit set as EOF unless we 
+	 * used unsigned here.
+	 */
 	VE_IGNORE_EINTR (bytes = read (fd, buf, 1));
 	if (bytes != 1)
 		return EOF;
@@ -1578,7 +1584,7 @@ gdm_hostent_free (GdmHostent *he)
 
 /* Like fopen with "w" */
 FILE *
-gdm_safe_fopen_w (const char *file)
+gdm_safe_fopen_w (const char *file, mode_t perm)
 {
 	int fd;
 	FILE *ret;
@@ -1592,7 +1598,7 @@ gdm_safe_fopen_w (const char *file)
 #ifdef O_NOFOLLOW
 			   |O_NOFOLLOW
 #endif
-			   , 0644);
+			   , perm);
 	} while G_UNLIKELY (errno == EINTR);
 	if (fd < 0)
 		return NULL;
@@ -1602,7 +1608,7 @@ gdm_safe_fopen_w (const char *file)
 
 /* Like fopen with "a+" */
 FILE *
-gdm_safe_fopen_ap (const char *file)
+gdm_safe_fopen_ap (const char *file, mode_t perm)
 {
 	int fd;
 	FILE *ret;
@@ -1630,7 +1636,7 @@ gdm_safe_fopen_ap (const char *file)
 #ifdef O_NOFOLLOW
 				   |O_NOFOLLOW
 #endif
-				   , 0644);
+				   , perm);
 		} while G_UNLIKELY (errno == EINTR);
 	}
 	if (fd < 0)
