@@ -44,7 +44,7 @@
  * if this module is ever moved outside of gdm.
  */
 
-#define CONFIGFILE EXPANDED_SYSCONFDIR "/gdm/modules/AccessDwellMouseEvents"
+#define CONFIGFILE GDMCONFDIR "/modules/AccessDwellMouseEvents"
 #define iseol(ch)       ((ch) == '\r' || (ch) == '\f' || (ch) == '\0' || \
                         (ch) == '\n')
 
@@ -396,7 +396,7 @@ parse_line(gchar *buf)
 		tmp_binding->timeout = timeout;
 	}
 
-	/* Find servcice. Permit blank space so arguments can be supplied. */
+	/* Find service. Permit blank space so arguments can be supplied. */
 	keyservice = strtok (NULL, "\n\r\f");
 	if (keyservice == NULL) {
 		/* TODO - Add an error message */
@@ -635,14 +635,30 @@ leave_enter_emission_hook (GSignalInvocationHint        *ihint,
 
 		for (act_li=curr_binding->actions; act_li != NULL; act_li=act_li->next) {
 			gchar *action = (gchar *)act_li->data;
+			gchar *ctrun;
 
 			g_return_val_if_fail (action != NULL, TRUE);
 
+#ifdef HAVE_CTRUN
+			ctrun = g_strdup_printf (
+				"/bin/sh -c \"/usr/bin/ctrun -l child -i none %s\"",
+				action);
+			if (!g_shell_parse_argv (ctrun, NULL, &argv, NULL)) {
+				g_free (ctrun);
+				continue;
+			}
+				
+			g_free (ctrun);
+#else
 			if (!g_shell_parse_argv (action, NULL, &argv, NULL))
 				continue;
+#endif
 
 			envp = get_exec_environment (gtk_window_get_screen(window));
 
+			if (debug_gestures)
+				syslog (LOG_WARNING, "Action is %s", action);
+
 			retval = g_spawn_async (NULL, argv, envp,
 				G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, NULL);
 
